@@ -164,7 +164,7 @@ class Response(Storage):
         if view:
             import cStringIO
             (obody, oview) = (self.body, self.view)
-            (self.body, self.view) = (cStringIO.StringIO(), view)            
+            (self.body, self.view) = (cStringIO.StringIO(), view)
             run_view_in(self._view_environment)
             page = self.body.getvalue()
             self.body.close()
@@ -339,16 +339,20 @@ class Session(Storage):
                 try:
                     response.session_file = \
                         open(response.session_filename, 'rb+')
-                    portalocker.lock(response.session_file,
-                            portalocker.LOCK_EX)
-                    response.session_locked = True
-                    self.update(cPickle.load(response.session_file))
-                    response.session_file.seek(0)
-                    oc = response.session_filename.split('/')[-1].split('-')[0]
-                    if check_client and client!=oc:
-                        raise Exception, "cookie attack"
+                    try:
+                        portalocker.lock(response.session_file,
+                                portalocker.LOCK_EX)
+                        response.session_locked = True
+                        self.update(cPickle.load(response.session_file))
+                        response.session_file.seek(0)
+                        oc = response.session_filename.split('/')[-1].split('-')[0]
+                        if check_client and client!=oc:
+                            raise Exception, "cookie attack"
+                    finally:
+                        pass
+                        #This causes admin login to break. Must find out why.
+                        #self._close(response)
                 except:
-                    self._close(response)
                     response.session_id = None
             if not response.session_id:
                 uuid = web2py_uuid()
@@ -511,4 +515,5 @@ class Session(Storage):
                 del response.session_file
             except:
                 pass
-            
+
+
