@@ -28,7 +28,8 @@ import socket
 import tempfile
 import random
 import string
-from fileutils import abspath
+import platform
+from fileutils import abspath, write_file
 from settings import global_settings
 from admin import add_path_first, create_missing_folders, create_missing_app_folders
 from globals import current
@@ -36,6 +37,9 @@ from globals import current
 from custom_import import custom_import_install
 from contrib.simplejson import dumps
 
+if platform.system=='Windows':
+    os.environ["PYTHONCASEOK"]="1"
+
 #  Remarks:
 #  calling script has inserted path to script directory into sys.path
 #  applications_parent (path to applications/, site-packages/ etc)
@@ -387,7 +391,7 @@ def wsgibase(environ, responder):
                 # ##################################################
 
                 http_host = request.env.http_host.split(':',1)[0]
-                
+
                 local_hosts = [http_host,'::1','127.0.0.1','::ffff:127.0.0.1']
                 if not global_settings.web2py_runtime_gae:
                     local_hosts += [socket.gethostname(),
@@ -688,7 +692,7 @@ def appfactory(wsgiapp=wsgibase,
             if not logfilename:
                 sys.stdout.write(line)
             elif isinstance(logfilename, str):
-                open(logfilename, 'a').write(line)
+                write_file(logfilename, line, 'a')
             else:
                 logfilename.write(line)
         except:
@@ -789,9 +793,7 @@ class HttpServer(object):
             signal.signal(signal.SIGINT, lambda a, b, s=self: s.stop())
         except:
             pass
-        fp = open(self.pid_filename, 'w')
-        fp.write(str(os.getpid()))
-        fp.close()
+        write_file(self.pid_filename, str(os.getpid()))
         self.server.start()
 
     def stop(self, stoplogging=False):
@@ -804,3 +806,4 @@ class HttpServer(object):
             os.unlink(self.pid_filename)
         except:
             pass
+
